<?php
/**
 * The Paper Tape Project -- Documentation
 *
 * This is a very small "template" like system that quickly wraps a design 
 * around almost every file (html,txt,source codes,...) in the Paper-Tape-Project
 * repository.
 * It should handle links, etc. well and works with PATH_INFO; typical calls are
 * linke "path/to/documentation/index.php/path/to/file.htm".
 * There's a small menu nested as PHP array in this script.
 *
 * Use this small script as in the public domain.
 * 22.12.2008, 12.02.2009 -- Sven Koeppel
 **/

  // the web site root path, taken relatively to the
  // global project path
  $doc = 'documentation';

  // where to get the file to display: Can be simply
  // $filename = $_SERVER['QUERY_STRING'];
  // or path info (advantage that links will work):
  $filename = substr($_SERVER['PATH_INFO'], 1); # strip trailing slash: /a/b => a/b

  // show default page if no one is given:
  $default_page = $doc.'/start.htm';
  #if(empty($filename)) { header("Location: $_SERVER[PHP_SELF]/$default_page"); exit; }
  if(empty($filename)) $filename = $default_page;

  // check if filename is correct
  $jail_dir = realpath('../'); # don't display files higher than parental dir
  $file_path = realpath('../'.$filename);
  $extension = substr(strrchr($file_path, '.'),1);
  $good_extensions = array('c', 'cpp', 'pl', 'htm', 'txt');

  #var_dump($extension, $filename, $file_path, $jail_dir, $good_extensions, in_array($extension, $good_extensions)); exit();
  if(0 !== strncmp($file_path, $jail_dir, strlen($jail_dir))
     || !in_array($extension, $good_extensions)
     || !is_readable('../'.$filename) ) {
      // user want's display file higher than parental dir
      // or file is not a nice file (like *.htm) or file is not
      // readable, then Redirect to file! Apache will handle error ;-)
      header("Location: $_SERVER[SCRIPT_NAME]/../../$filename");
      echo "Won't make file $filename beautiful\nRefering to that file...\n";
      exit;
  }

  // do a little bit lightweight caching:
  // don't calculate etag since that's too expensive
  $last_modified_time = max(
      filemtime("../$filename"), filemtime($_SERVER['SCRIPT_FILENAME'])
  ); 
  #$etag = md5_file("../$filename"); 

  header("Last-Modified: ".gmdate("D, d M Y H:i:s", $last_modified_time)." GMT"); 
  #header("Etag: $etag"); 

  if (@strtotime($_SERVER['HTTP_IF_MODIFIED_SINCE']) == $last_modified_time) {
     #|| trim($_SERVER['HTTP_IF_NONE_MATCH']) == $etag) { 
     header("HTTP/1.1 304 Not Modified"); 
     exit; 
  }

  // since PHP cannot initialize global variables on an intelligent way:
  $exec_action = $extension=='htm' ? 'display_file' : 'syntax_highlight_file';

  function display_file($filename) {
      // display HTML file.
      // parse file until <body> starts...
      $handle = fopen("../$filename", 'r');
      if(!$handle) { print "Error at opening $filename\n"; exit; }
      while(!feof($handle)) {
          if(strpos(fgets($handle), "<body") !== false)
              break;
      }
      fpassthru($handle);
  }

  function syntax_highlight_file($filename) {
      // syntax highlight some file
      global $extension; // yes, this is bad.
      echo "<h2>Contents of <code>$filename</code></h2>";
      if($extension == 'txt') {
          echo "<pre>";
          readfile("../$filename");
          echo "</pre>";
      } else {
          include_once('geshi.php');
          $geshi = new GeSHi();
          $geshi->load_from_file("../$filename");
          $geshi->enable_line_numbers(GESHI_NORMAL_LINE_NUMBERS);
          echo $geshi->parse_code();
      }
  }
?>
<html>
<head>
  <title>The Punched Paper Project / <?=$filename; ?></title>
  <link rel="stylesheet" href="<?=$_SERVER['SCRIPT_NAME'].'/'.$doc; ?>/style.css" type="text/css">
  <meta http-equiv="content-type" content="text/html; charset=ISO-8859-1" />
</head>
<body>
<div id="header">
  <h1><a href="<?=$_SERVER['SCRIPT_NAME']; ?>">The Punched Paper Project</a></h1>
  <p class="subtitle"><a href="<?=$_SERVER['SCRIPT_NAME'].'/../../'.$filename; ?>" title="Read the original file">/<?=$filename; ?></a></p>
  <a href="http://dev.technikum29.de/" class="powered-by"><img src="<?=$_SERVER['SCRIPT_NAME'].'/'.$doc; ?>/src/powered-by-icon.png" alt="a project from the technikum29 development zone"></a>
</div>
<div id="navigation">
<?php

 $nav = array(
	#all URLs are taken relative to one subdir (e.g. documentation/).
	'Home' => $default_page,
	'The Paper Tape Project' => array(
		'Start' => $doc.'/paper-tape-project.htm',
		'Abstract' => array(
			'Subproject overview' => 'paper-tape/README.txt',
			'To Do' => 'paper-tape/TODO.txt',
			'Using Windows' => 'paper-tape/WINDOWS.txt',
		),
		'Data' => 'paper-tape/daten/README.htm',
		'Perl tools' => 'paper-tape/perl-tools/README.htm',
		'Labeling and Fonts' => array(
			'Generating Labels' => 'paper-tape/schriften/README.htm',
			'Font Files' => 'paper-tape/schriften/font_files.htm'
		),
		'Devices' => array(
			'Facit Tape Punch' => 'paper-tape/driver/FACIT-MANUAL.txt',
			'Ghielmetti Tape Reader' => 'paper-tape/reader/GHIELMETTI-MANUAL.txt'
		),
		'Driver framework' => array(
			'Legacy reader documentation' => 'paper-tape/reader/README.txt',
			'Legacy puncher documentation' => 'paper-tape/puncher/README.txt',
			'Legacy old puncher documentation' => 'paper-tape/userspace-driver/README.txt'
		),
		'Visualisation' => 'paper-tape/visualisator/README.htm',
		'Web Frontend' => array(
			'Abstract' => 'paper-tape/web-frontend/README.txt',
			'Generating Paper Tapes online!' => 'paper-tape/web-frontend/'
		)
	),
	'The Punch Card Project' => array(
		'Start' => $doc.'/punch-card.htm',
		'Device Drivers' => array(
			'Overview' => 'punch-card/driver/README.htm',
			'Nixdorf 0377.01' => 'punch-card/driver/nixdorf-0377.01/README.htm'
		)
	),
	#'About' => $doc.'/about.htm'
 );

 function print_nav($nav) {
    global $filename;
    print "<ul>"; foreach($nav as $name => $url) {
        echo "<li>";
        if(is_array($url)) {
           print "<em>$name</em>"; print_nav($url);
        } else
           print $url == $filename ? "<strong title=\"You're currently on that page\">$name</strong>" : "<a href=\"$_SERVER[SCRIPT_NAME]/$url\">$name</a>";
        echo "</li>";
    }
    print "</ul>";
 }
 print_nav($nav);
?>
</div>
<div id="content">
<?php
  call_user_func($exec_action, $filename);#, $exec_array);
?>
</div>
<div id="footer">
  This page was last modified at <?=date('r', $last_modified_time); ?>
  <br/>
  Further reading:
  <a href="http://dev.technikum29.de">dev.technikum29.de</a> |
  <a href="http://dev.technikum29.de/websvn/listing.php?repname=paper-tape-project&path=%2F&sc=0">Paper-Tape-Project WebSVN</a> |
  <a href="http://koeppel.homeunix.org">koeppel.homeunix.org</a>
</div>
